import { _decorator, assetManager } from 'cc';
const { ccclass, property } = _decorator;

@ccclass('WebZipLoader')
export class WebZipLoader {
    private static _instance: WebZipLoader = null;
    public static get instance() {
        if (!this._instance) {
            this._instance = new WebZipLoader();
        }
        return this._instance;
    }



    private zipCache = new Map<string, any>();
    private resCache = new Map();

    constructor() {
        this.injectXMLHttpRequest();
        this.injectDownloadImage('.png');
        this.injectDownloadImage('.jpg');
        this.injectDownloadImage('.bmp');
        this.injectDownloadImage('.jpeg');
        this.injectDownloadImage('.gif');
        this.injectDownloadImage('.ico');
        this.injectDownloadImage('.tiff');
        this.injectDownloadImage('.webp');
        this.injectDownloadImage('.image');

    }
    /**
  * Support the file formate was dwonloaded using XMLHttpRequest, like .cconb, .astc, .json.
  * @returns 
  */
    private injectXMLHttpRequest(): boolean {


        const that = this;


        {
            const originalSrcDescriptor = Object.getOwnPropertyDescriptor(HTMLScriptElement.prototype, 'src');

            Object.defineProperty(HTMLScriptElement.prototype, 'src', {
                get() {
                    return originalSrcDescriptor!.get!.call(this);
                },
                set(value: string) {
                    // 检查缓存
                    if (that.zipCache.has(value)) {
                        console.error('[zipCache.js]', value);
                        const scriptContent = that.zipCache.get(value)
                        const blob = new Blob([scriptContent.async('arraybuffer')], { type: 'application/javascript' });
                        const blobUrl = URL.createObjectURL(blob);
                        // 延迟设置 Blob URL
                        setTimeout(() => {
                            originalSrcDescriptor!.set!.call(this, blobUrl);
                        }, 50);

                        // 清理内存
                        this.addEventListener('load', () => URL.revokeObjectURL(blobUrl));
                        return;
                    }
                    // 无缓存则正常加载
                    originalSrcDescriptor!.set!.call(this, value);
                },
                configurable: true,
            });
        }


        const accessor = Object.getOwnPropertyDescriptor(XMLHttpRequest.prototype, 'response');
        if (accessor === undefined)
            return false;


        // 注入新 open 方法
        const oldOpen = (XMLHttpRequest.prototype).open;
        // @ts-ignore
        (XMLHttpRequest.prototype).open = function (method: string, url: string, async: boolean, user: string, password: string) {
            const extension = url.split('.').pop();
            if (extension === 'cconb' || extension === 'astc' || extension === 'json') {

            }

            // 檢查zip 加載時是否有這個url
            if (that.zipCache.has(url)) {
                this.zipCacheUrl = url;
            } else {
                this.zipCacheUrl = null;
                // console.error(`[${that.constructor.name}] The requested asset not in cache: ${url}`);
            }

            return oldOpen.apply(this, [method, url, async, user, password]);
        }

        // 注入新 send 方法
        const oldSend = (XMLHttpRequest.prototype).send;
        (XMLHttpRequest.prototype).send = async function (data) {
            if (this.zipCacheUrl) {
                if (!that.resCache.has(this.zipCacheUrl)) {
                    const cache = that.zipCache.get(this.zipCacheUrl);
                    if (cache) {
                        if (this.responseType === "json") {
                            const text = await cache.async("text");
                            that.resCache.set(this.zipCacheUrl, text);
                        } else {
                            const res = await cache.async(this.responseType);
                            that.resCache.set(this.zipCacheUrl, res);
                        }
                        that.zipCache.delete(this.zipCacheUrl);
                    }
                }

                if (typeof this.onload === "function") {
                    const event = new ProgressEvent('load', {
                        lengthComputable: false,
                        loaded: 0,
                        total: 0
                    });
                    this.onload(event);
                }
                return;
            }
            return oldSend.apply(this, [data]);
        }

        Object.defineProperty((XMLHttpRequest.prototype), 'response', {
            get: function () {
                if (this.zipCacheUrl) {
                    const res = that.resCache.get(this.zipCacheUrl);
                    return this.responseType === "json" ? JSON.parse(res) : res;
                }
                return accessor.get ? accessor.get.call(this) : undefined;
            },
            set: function () { },
            configurable: true
        });

        return true;
    }

    private injectDownloadImage(extension: string) {
        assetManager.downloader.register(extension, async (url, options, onComplete) => {
            if (this.resCache.has(url)) {
                const res = this.resCache.get(url);
                onComplete(null, res);
            } else {
                if (this.zipCache.has(url)) {
                    const cache = this.zipCache.get(url);
                    if (cache) {
                        const res = await cache.async("blob");
                        this.resCache.set(url, res);
                        this.zipCache.delete(url);
                        onComplete(null, res);
                        return;
                    }
                }
                const img = new Image();
                img.crossOrigin = 'anonymous';
                img.onload = () => { onComplete(null, img); };
                img.onerror = (e) => { onComplete(new Error(e instanceof Event ? e.type : e), null); };
                img.src = url;
            }
        });
    }

    get JSZip() {
        return new window["JSZip"]()
    }

    downloadZip(url: string, bundleName: string, bundleVersion): Promise<void> {
        return new Promise((resolve, reject) => {
            fetch(`${url}/${bundleName}.${bundleVersion}.zip`)
                .then(response => {
                    if (!response.ok) {
                        throw new Error(`HTTP error! Status: ${response.status}`);
                    }
                    return response.arrayBuffer();
                })
                .then(buffer => {
                    return this.JSZip.loadAsync(buffer); // 依赖JSZip返回Promise
                })
                .then(zipFile => {
                    // 遍历ZIP文件并写入缓存
                    zipFile.forEach((relativePath, fileEntry) => {
                        if (fileEntry.dir) return;
                        this.zipCache.set(`${url}/${bundleName}/${relativePath}`, fileEntry);
                    });
                    resolve(); // 所有操作完成
                })
                .catch(e => {
                    console.error(`[${this.constructor.name}].downloadZip`, e);
                    reject(e); // 明确传递错误
                });
        });
    }



    getMimeTypeFromPath(filePath) {
        const extension = filePath.split('.').pop().toLowerCase();
        let mimeType = '';

        switch (extension) {
            case 'mp3':
                mimeType = 'audio/mpeg';
                break;
            case 'jpg':
            case 'jpeg':
                mimeType = 'image/jpeg';
                break;
            case 'png':
                mimeType = 'image/png';
                break;
            case 'gif':
                mimeType = 'image/gif';
                break;
            case 'json':
                mimeType = 'application/json';
                break;
            case 'ttf':
                mimeType = 'application/octet-stream';
                break;
            // 添加其他常见文件扩展名的处理逻辑
            default:
                mimeType = '';
                break;
        }
        return mimeType;
    }


}


